import requests
from enum import Enum
from urllib.parse import urlencode
from typing import (
    Optional,
    List
)

from everart.util import (
    make_url,
    APIVersion,
    EverArtError
)
from everart.client_interface import ClientInterface

class ModelStatus(Enum):
    PENDING = 'PENDING'
    PROCESSING = 'PROCESSING'
    TRAINING = 'TRAINING'
    READY = 'READY'
    FAILED = 'FAILED'
    CANCELED = 'CANCELED'

class Model:
    def __init__(self, id: str, name: str, status: ModelStatus):
        self.id = id
        self.name = name
        self.status = status

class ModelsFetchResponse:
    def __init__(self, models: List[Model], has_more: bool):
        self.models = models
        self.has_more = has_more

class Models():
    
    def __init__(
        self,
        client: ClientInterface
    ) -> None:
        self.client = client
  
    def fetch(
        self,
        before_id: Optional[str] = None,
        limit: Optional[int] = None,
        search: Optional[str] = None,
        status: Optional[ModelStatus] = None
    ) -> ModelsFetchResponse:        
        params = {}
        if before_id:
            params['before_id'] = before_id
        if limit:
            params['limit'] = limit
        if search:
            params['search'] = search
        if status:
            params['status'] = status.value
        
        endpoint = "models"
        if params:
            endpoint += '?' + urlencode(params)

        response = requests.get(
            make_url(APIVersion.V1, endpoint),
            headers=self.client.headers
        )

        if response.status_code == 200 \
            and isinstance(response.json().get('models'), list) \
            and isinstance(response.json().get('has_more'), bool):
            models = [Model(**model) for model in response.json().get('models')]
            return ModelsFetchResponse(models, response.json().get('has_more'))

        raise EverArtError(
            response.status_code,
            'Failed to get models',
            response.json()
        )


import pytest
import everart

def test_client():
    client = everart.default_client
    assert client is not None

@pytest.fixture
def test_fetch_models():
    result = everart.v1.models.fetch(limit=1)
    assert result is not None
    assert len(result.models) == 1
    return result.models[0]

@pytest.fixture
def test_create_prediction(test_fetch_models):
    model = test_fetch_models
    predictions = everart.v1.predictions.create(
        model_id=model.id, 
        prompt="a test image of a model", 
        type=everart.PredictionType.TXT_2_IMG,
        image_count=1
    )
    assert predictions is not None
    assert len(predictions) == 1
    return predictions[0]

def test_fetch_prediction(test_create_prediction):
    prediction_id = test_create_prediction.id
    prediction = everart.v1.predictions.fetch(id=prediction_id)
    assert prediction is not None
    assert prediction.id == prediction_id
    assert prediction.status in {
        everart.PredictionStatus.STARTING.value, 
        everart.PredictionStatus.PROCESSING.value, 
        everart.PredictionStatus.SUCCEEDED.value, 
        everart.PredictionStatus.FAILED.value,
        everart.PredictionStatus.CANCELED.value
    }

def test_fetch_prediction_with_polling(test_create_prediction):
    prediction_id = test_create_prediction.id
    prediction = everart.v1.predictions.fetch_with_polling(id=prediction_id)
    assert prediction is not None
    assert prediction.status == everart.PredictionStatus.SUCCEEDED.value
    assert prediction.image_url is not None

def test_create_prediction_with_polling(test_fetch_models):
    model = test_fetch_models
    prediction = everart.v1.predictions.create_with_polling(
        model_id=model.id, 
        prompt="a test image of a model", 
        type=everart.PredictionType.TXT_2_IMG,
    )
    assert prediction is not None
    assert prediction.status == everart.PredictionStatus.SUCCEEDED.value
    assert prediction.image_url is not None


import requests
import time
from enum import Enum
from typing import (
    Optional,
    List
)

from everart.util import (
    make_url,
    APIVersion,
    EverArtError
)
from everart.client_interface import ClientInterface

class PredictionStatus(Enum):
    STARTING = 'STARTING'
    PROCESSING = 'PROCESSING'
    SUCCEEDED = 'SUCCEEDED'
    FAILED = 'FAILED'
    CANCELED = 'CANCELED'

class PredictionType(Enum):
    TXT_2_IMG = 'txt2img'

class Prediction:
    def __init__(
        self, 
        id: str, 
        model_id: str,
        status: PredictionStatus,
        image_url: Optional[str],
        type: PredictionType
    ):
        self.id = id
        self.model_id = model_id
        self.status = status
        self.image_url = image_url
        self.type = type

from everart.client_interface import ClientInterface

class Predictions():
    
    def __init__(
        self,
        client: ClientInterface
    ) -> None:
        self.client = client
  
    def fetch(
        self,
        id: str
    ) -> Prediction:
        endpoint = "predictions/" + id

        response = requests.get(
            make_url(APIVersion.V1, endpoint),
            headers=self.client.headers
        )
        
        if response.status_code == 200 \
            and isinstance(response.json().get('prediction'), dict):
            return Prediction(**response.json().get('prediction'))

        raise EverArtError(
            response.status_code,
            'Failed to get prediction',
            response.json()
        )
    
    def is_prediction_finalized(
        self,
        prediction: Prediction
    ) -> bool:
        return prediction.status in {PredictionStatus.SUCCEEDED.value, PredictionStatus.FAILED.value, PredictionStatus.CANCELED.value}
    
    def fetch_with_polling(
        self,
        id: str
    ) -> Prediction:
        prediction = self.fetch(id)

        time_elapsed = 0

        while self.is_prediction_finalized(prediction) is False:
            prediction = self.fetch(prediction.id)
            if self.is_prediction_finalized(prediction) is True:
                break
            if time_elapsed >= 240:
                raise Exception("Prediction took too long to finalize")
            time_elapsed += 5
            time.sleep(5)

        return prediction
  
    def create(
        self,
        model_id: str,
        prompt: str,
        type: PredictionType,
        image_count: Optional[int] = None,
        height: Optional[int] = None,
        width: Optional[int] = None
    ) -> List[Prediction]:
        body = {
            'prompt': prompt,
            'type': type.value
        }

        if image_count:
            body['image_count'] = image_count
        if height:
            body['height'] = height
        if width:
            body['width'] = width

        endpoint = "models/" + model_id + "/predictions"

        response = requests.post(
            make_url(APIVersion.V1, endpoint),
            json=body,
            headers=self.client.headers
        )
        
        if response.status_code == 200 \
            and isinstance(response.json().get('predictions'), list):
            return [Prediction(**model) for model in response.json().get('predictions')]

        raise EverArtError(
            response.status_code,
            'Failed to get prediction',
            response.json()
        )
    
    def create_with_polling(
        self,
        model_id: str,
        prompt: str,
        type: PredictionType,
        height: Optional[int] = None,
        width: Optional[int] = None
    ) -> Prediction:
        predictions = self.create(
            model_id=model_id,
            prompt=prompt,
            type=type,
            image_count=1,
            height=height,
            width=width
        )

        if not predictions or len(predictions) == 0:
            raise Exception("No predictions created")
        
        prediction = predictions[0]

        prediction = self.fetch_with_polling(prediction.id)

        return prediction


import json
from enum import Enum

EVERART_BASE_URL = "https://api.everart.ai"

class APIVersion(Enum):
    V1 = "v1"

class OutputFormat(Enum):
    JPEG = "jpeg"
    PNG = "png"
    WEBP = "webp"

def make_url(
    version: APIVersion,
    endpoint: str
) -> str:
    return f"{EVERART_BASE_URL}/{version.value}/{endpoint}"

class EverArtErrorName(Enum):
    INVALID_REQUEST_ERROR = 'EverArtInvalidRequestError'
    UNAUTHORIZED_ERROR = 'EverArtUnauthorizedError'
    CONTENT_MODERATION_ERROR = 'EverArtContentModerationError'
    RECORD_NOT_FOUND_ERROR = 'EverArtRecordNotFoundError'
    UNKNOWN_ERROR = 'EverArtUnknownError'

class EverArtError(Exception):
    def __init__(self, status: int, message: str, data: any = None):
        try:
            data_message = json.dumps(data)
        except:
            data_message = ''

        full_message = f"{message}: {data_message}"
        super().__init__(full_message)

        name = EverArtErrorName.UNKNOWN_ERROR

        if status == 400:
            name = EverArtErrorName.INVALID_REQUEST_ERROR
        elif status == 401:
            name = EverArtErrorName.UNAUTHORIZED_ERROR
        elif status == 403:
            name = EverArtErrorName.CONTENT_MODERATION_ERROR
        elif status == 404:
            name = EverArtErrorName.RECORD_NOT_FOUND_ERROR

        self.name = name


from dotenv import load_dotenv
import os

from everart.client import Client
from everart.models import (
    ModelStatus,
    Model,
    ModelsFetchResponse
)
from everart.predictions import (
    PredictionStatus,
    PredictionType,
    Prediction
)

ModelStatus = ModelStatus
Model = Model
ModelsFetchResponse = ModelsFetchResponse
PredictionStatus = PredictionStatus
PredictionType = PredictionType
Prediction = Prediction

load_dotenv()

api_key = os.environ.get("EVERART_API_KEY")

default_client = Client(api_key=api_key)

v1 = default_client.v1


from everart.client_interface import ClientInterface
from everart.models import Models
from everart.predictions import Predictions

class V1:
    def __init__(
        self,
        client: ClientInterface
    ) -> None:
        self.client = client

    @property
    def models(self):
        return Models(client=self.client)

    @property
    def predictions(self):
        return Predictions(client=self.client)

class Client(ClientInterface):
    
    def __init__(
        self,
        api_key: str
    ) -> None:
        if api_key is None:
            raise ValueError("API key cannot be None")
        
        self.api_key = api_key

    @property
    def headers(self):
        return {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json"
        }

    @property
    def v1(self):
        return V1(client=self)